@@ -229,12 +229,10 @@ class MVForecaster:
         Returns:
             None
 
-        >>> from sklearn.linear_model import Lasso
-        >>> mvf.add_sklearn_estimator(Lasso,called='lasso')
-        >>> mvf.set_estimator('lasso')
-        >>> mvf.ingest_grid({'alpha':[.1,.5,1,1.5,2],'lags':[1,2,3]})
-        >>> mvf.tune()
-        >>> mvf.auto_forecast()
+        >>> from sklearn.ensemble import StackingRegressor
+        >>> mvf.add_sklearn_estimator(StackingRegressor,called='stacking')
+        >>> mvf.set_estimator('stacking')
+        >>> mvf.manual_forecast(...)
         """
         globals()[called + '_'] = imported_module
         _sklearn_imports_[called] = globals()[called + '_']
@@ -454,10 +452,12 @@ class MVForecaster:
                     fcster=self.estimator,
                     tune=True,
                     dynamic_testing=dynamic_tuning,
-                    **hp
+                    **hp,
                 )
                 for s, l in zip(series,labels):
-                    metrics[l + '_metric'].append(globals()[self.validation_metric](val_ac[s],val_preds[s]))
+                    vp = val_preds[s]
+                    va = val_ac[s][-len(vp):]
+                    metrics[l + '_metric'].append(globals()[self.validation_metric](va,vp))
             except TypeError:
                 raise
             except Exception as e:
@@ -467,6 +467,7 @@ class MVForecaster:
                 )
         metrics = pd.DataFrame(metrics)
         if metrics.shape[0] > 0:
+            self.dynamic_tuning = dynamic_tuning
             self.grid.reset_index(drop=True,inplace=True)
             self.grid_evaluated = self.grid.copy()
             self.grid_evaluated["validation_length"] = self.validation_length
@@ -475,21 +476,125 @@ class MVForecaster:
                 metrics['optimized_metric'] = metrics.apply(_optimizer_funcs_[self.optimize_on],axis=1)
             else:
                 metrics['optimized_metric'] = metrics[self.optimize_on + '_metric']
-
             self.grid_evaluated = pd.concat([self.grid_evaluated,metrics],axis=1)
+        else:
+            self.grid_evaluated = pd.DataFrame()
+        self._find_best_params(self.grid_evaluated)
+
+    def cross_validate(self,k=5,rolling=False,dynamic_tuning=False):
+        """ tunes a model's hyperparameters using time-series cross validation. 
+        monitors the metric specified in the valiation_metric attribute. 
+        set an estimator before calling. this is an alternative method to tune().
+        reads a grid for the estimator from MVGrids.py unless a grid is ingested manually. 
+        each fold size is equal to one another and is determined such that the last fold's 
+        training and validation sizes are the same (or close to the same). with rolling = True, 
+        all train sizes will be the same for each fold. 
+
+        Args:
+            k (int): default 5. the number of folds. must be at least 2.
+            rolling (bool): default False. whether to using a rolling method.
+            dynamic_tuning (bool): default False.
+
+        Returns:
+            None
+        """
+        rolling = bool(rolling)
+        k = int(k)
+        descriptive_assert(k>=2,ValueError,f'k must be at least 2, got {k}')
+        mvf = self.__deepcopy__()
+        usable_obs = len(mvf.series1['y']) - mvf.test_length
+        val_size = usable_obs // (k+1)
+        mvf.set_validation_length(val_size)
+        grid_evaluated_cv = pd.DataFrame()
+        for i in range(k):
+            if i > 0:
+                mvf.current_xreg = {k:pd.Series(v.values[:-val_size]) for k, v in mvf.current_xreg.items()}
+                mvf.current_dates = pd.Series(mvf.current_dates.values[:-val_size])
+                for s in range(mvf.n_series):
+                    setattr(
+                        mvf,
+                        f'series{s+1}',
+                        {
+                            'y':pd.Series(getattr(mvf,f'series{s+1}')['y'].values[:-val_size]),
+                            'levely':getattr(mvf,f'series{s+1}')['levely'][:-val_size],
+                            'integration':getattr(mvf,f'series{s+1}')['integration'],
+                        }
+                    )
+            
+            mvf2 = mvf.__deepcopy__()
+            if rolling:
+                n = val_size*2+mvf2.test_length
+                mvf2.current_dates = pd.Series(mvf2.current_dates.values[-n:])
+                mvf2.current_xreg = {k:pd.Series(v.values[-n:]) for k,v in mvf2.current_xreg.items()}
+                for s in range(mvf2.n_series):
+                    setattr(
+                        mvf2,
+                        f'series{s+1}',
+                        {
+                            'y':pd.Series(getattr(mvf2,f'series{s+1}')['y'].values[-n:]),
+                            'levely':getattr(mvf2,f'series{s+1}')['levely'][-n:],
+                            'integration':getattr(mvf2,f'series{s+1}')['integration'],
+                        }
+                    )
+
+            mvf2.tune(dynamic_tuning=dynamic_tuning)
+            orig_grid = mvf2.grid.copy()
+            if mvf2.grid_evaluated.shape[0] == 0:
+                self.grid = pd.DataFrame()
+                self._find_best_params(grid_evaluated_cv)
+                return # writes a warning and moves on
+        
+            mvf2.grid_evaluated['fold'] = i
+            mvf2.grid_evaluated['rolling'] = rolling
+            mvf2.grid_evaluated['train_length'] = len(mvf2.series1['y']) - val_size - mvf2.test_length
+            grid_evaluated_cv = pd.concat([grid_evaluated_cv,mvf2.grid_evaluated])
+
+        # convert into str for the group or it fails
+        if 'lags' in grid_evaluated_cv:
+            grid_evaluated_cv['lags'] = grid_evaluated_cv['lags'].apply(lambda x: str(x))
+
+        grid_evaluated = grid_evaluated_cv.groupby(
+            grid_evaluated_cv.columns.to_list()[:-(4 + mvf2.n_series)],
+            dropna=False,
+        )['optimized_metric'].mean().reset_index()
+
+        # convert back to whatever it was before or it fails when calling the hyperparam vals
+        # contributions welcome for a more elegant solution
+        if 'lags' in grid_evaluated:
+            grid_evaluated['lags'] = grid_evaluated['lags'].apply(lambda x: eval(x))
+
+        self.grid = grid_evaluated.iloc[:,:-3]
+        self.dynamic_tuning = mvf2.dynamic_tuning
+        self._find_best_params(grid_evaluated)
+        self.grid_evaluated = grid_evaluated_cv.reset_index(drop=True)
+        self.grid = orig_grid # because None changes to np.nan otherwise
+        self.cross_validated = True
+
+    def _find_best_params(self,grid_evaluated):
+        self.cross_validated = False # will be changed to True appropriately if cv was called
+        if grid_evaluated.shape[0] > 0:
             if self.validation_metric == "r2":
                 best_params_idx = self.grid.loc[
-                    self.grid_evaluated["optimized_metric"]
-                    == self.grid_evaluated["optimized_metric"].max()
+                    grid_evaluated["optimized_metric"]
+                    == grid_evaluated["optimized_metric"].max()
                 ].index.to_list()[0]
             else:
                 best_params_idx = self.grid.loc[
-                    self.grid_evaluated["optimized_metric"]
-                    == self.grid_evaluated["optimized_metric"].min()
+                    grid_evaluated["optimized_metric"]
+                    == grid_evaluated["optimized_metric"].min()
                 ].index.to_list()[0]
                 
             self.best_params = {k: v[best_params_idx] for k, v in self.grid.to_dict(orient='series').items()}
-            self.validation_metric_value = self.grid_evaluated.loc[
+            self.best_params = {
+                k: (
+                    v 
+                    if not isinstance(v,float) 
+                    else int(v) if str(v).endswith('.0') else None if np.isnan(v) 
+                    else v
+                ) 
+                for k,v in self.best_params.items()
+            }
+            self.validation_metric_value = grid_evaluated.loc[
                 best_params_idx, "optimized_metric"
             ]
         else:
@@ -497,7 +602,6 @@ class MVForecaster:
                 f"none of the keyword/value combos stored in the grid could be evaluated for the {self.estimator} model"
             )
             self.best_params = {}
-        self.dynamic_tuning = dynamic_tuning
 
     def auto_forecast(self, call_me=None, dynamic_testing=True):
         """ auto forecasts with the best parameters indicated from the tuning process.
@@ -528,7 +632,8 @@ class MVForecaster:
         call_me = self.estimator if call_me is None else call_me
         if len(self.best_params) > 0:
             self.history[call_me]['Tuned'] = True if not self.dynamic_tuning else 'Dynamically'
-            self.history[call_me]['ValidationSetLength'] = self.validation_length
+            self.history[call_me]['CrossValidated'] = self.cross_validated
+            self.history[call_me]['ValidationSetLength'] = self.validation_length if not self.cross_validated else np.nan
             self.history[call_me]['ValidationMetric'] = self.validation_metric
             self.history[call_me]['ValidationMetricValue'] = self.validation_metric_value
             self.history[call_me]['grid_evaluated'] = self.grid_evaluated
@@ -571,14 +676,18 @@ class MVForecaster:
     def tune_test_forecast(
         self,
         models,
+        cross_validate=False,
         dynamic_tuning=False,
-        dynamic_testing=True
+        dynamic_testing=True,
+        **cvkwargs,
     ):
         """ iterates through a list of models, tunes them using grids in MVGrids.py, and forecasts them.
 
         Args:
-            models (list-like):
-                each element must be in _can_be_tuned_.
+            models (list-like): the models to iterate through.
+            cross_validate (bool): default False
+                whether to tune the model with cross validation. 
+                if False, uses the validation slice of data to tune.
             dynamic_tuning (bool): default False.
                 whether to dynamically tune the forecast (meaning AR terms will be propogated with predicted values).
                 setting this to False means faster performance, but gives a less-good indication of how well the forecast will perform out x amount of periods.
@@ -587,6 +696,7 @@ class MVForecaster:
                 whether to dynamically test the forecast (meaning AR terms will be propogated with predicted values).
                 setting this to False means faster performance, but gives a less-good indication of how well the forecast will perform out x amount of periods.
                 when False, test-set metrics effectively become an average of one-step forecasts.
+            **cvkwargs: passed to the cross_validate() method.
 
         Returns:
             None
@@ -601,7 +711,10 @@ class MVForecaster:
         )
         for m in models:
             self.set_estimator(m)
-            self.tune(dynamic_tuning=dynamic_tuning)
+            if cross_validate:
+                self.cross_validate(dynamic_tuning=dynamic_tuning,**cvkwargs)
+            else:
+                self.tune(dynamic_tuning=dynamic_tuning)
             self.auto_forecast(dynamic_testing=dynamic_testing)
 
     def set_optimize_on(self, how):
@@ -894,6 +1007,7 @@ class MVForecaster:
             "FittedVals": fitted_vals,
             "Resids": resids,
             "Tuned": None,
+            "CrossValidated": False,
             "DynamicallyTested": self.dynamic_testing,
             "TestSetLength": self.test_length,
             "TestSetPredictions": test_set_preds,
@@ -948,10 +1062,11 @@ class MVForecaster:
             )
             models_metrics = {m: v[determine_best_by] for m, v in self.history.items()}
 
-            if self.optimize_on == 'mean':
-                models_metrics = {m:np.mean(list(v.values())) for m, v in models_metrics.items()}
+            if self.optimize_on in _optimizer_funcs_:
+                models_metrics = {m:_optimizer_funcs_[self.optimize_on](list(v.values())) for m, v in models_metrics.items()}
             else:
-                models_metrics = {m:v['y{}'.format(self.optimize_on.split('series')[-1])] for m, v in models_metrics.items()}
+                series, label  = self._parse_series(self.optimize_on)
+                models_metrics = {m:v[series[0]] for m, v in models_metrics.items()}
 
             x = [h[0] for h in Counter(models_metrics).most_common()] 
             self.best_model = (
@@ -1216,6 +1331,7 @@ class MVForecaster:
             "Scaler",
             "Observations",
             "Tuned",
+            "CrossValidated",
             "DynamicallyTested",
             "Integration",
             "TestSetLength",
@@ -1266,12 +1382,11 @@ class MVForecaster:
                     elif c == "LastTestSetActual":
                         model_summary_sm[c] = [self.history[m]["TestSetActuals"][s][-1]]
                     elif c == "OptimizedOn" and hasattr(self,'best_model'): 
-                        if self.optimize_on == 'mean':
-                            model_summary_sm['OptimizedOn'] = ['mean']
-                        elif hasattr(self,'series_name_map'):
-                            model_summary_sm['OptimizedOn'] = [self.series_name_map[self.optimize_on]]
-                        else:
+                        if self.optimize_on in _optimizer_funcs_:
                             model_summary_sm['OptimizedOn'] = [self.optimize_on]
+                        else:
+                            series, label = self._parse_series(self.optimize_on)
+                            model_summary_sm['OptimizedOn'] = [label]
                         if hasattr(self,'optimize_metric'):
                             model_summary_sm['MetricOptimized'] = self.optimize_metric
                         model_summary_sm["best_model"] = m == self.best_model
@@ -1483,7 +1598,6 @@ class MVForecaster:
         metric_results['mean'] = metric_results.mean(axis=1)
         self.history[model]['BacktestMetrics'] = metric_results
         self.history[model]['BacktestValues'] = value_results
-        # TODO: value_results can be a dataframe subclass with a custom plot() method
     
     def export_backtest_metrics(self,model):
         """ extracts the backtest metrics for a given model.
